/*******************************************************************************
  Sensors

  Functions for sensors and acquisition.

  ------------------------------------------------------------------------------
  (c) Copyright 2022 University of Liege
  Author: Thomas Lambert <t.lambert@uliege.be>
  ULiege - Aeroelasticity and Experimental Aerodynamics
  MIT License
  Repo: https://gitlab.uliege.be/thlamb/mecharaptor-controller
*******************************************************************************/

// Read 12-bit rotary encoder
// Inspired from RobinL modified by linarism on forum.arduino.cc
unsigned int ReadSensor(uint8_t csn, int8_t orient, int8_t offset) {
  unsigned int ret = 0;

  digitalWrite(csn, LOW);
  delayMicroseconds(1);  //Waiting for Tclkfe

  //Passing 12 times, from 0 to 11
  for (int x = 0; x < 12; x++) {
    digitalWrite(CLK_PIN, LOW);
    delayMicroseconds(1);  //Tclk/2
    digitalWrite(CLK_PIN, HIGH);
    delayMicroseconds(1);                    //Tdo valid, like Tclk/2
    ret = (ret << 1) | digitalRead(DO_PIN);  //shift all the entering data to the left and past the pin state to it. 1e bit is MSB
  }

  digitalWrite(csn, HIGH);  //deselects the encoder from reading

  return (1 - orient) / 2 * ENC_PRECISION + orient * ret - (offset * ENC_PRECISION / 2);
}

// Get the tension and current of a PSU
void GetPsu(byte vol_pin, byte cur_pin, float &vol, float &cur) {
  vol = GetTension(vol_pin);
  cur = GetCurrent(cur_pin);
}

// Measure current consumed by module
// From https://www.youtube.com/watch?v=SiHfjzcqnU4
float GetCurrent(byte cur_pin) {
  float volt_raw = (5.0 / 1023.0) * analogRead(cur_pin);

  float volt = volt_raw - gCUR_QOV + SEN0098_OFFSET;  // Trimming value to make current equal to 0 when no current
  return volt / gCUR_FACT;                            // Actual current
}

// Measure the tension provided by the PSU using the voltage divider
float GetTension(byte vol_pin) {
  float vol_raw = analogRead(vol_pin) * (5.0 / 1023.0);

  return vol_raw * (DIV_RES1 + DIV_RES2) / (DIV_RES2);
}
